/* Main.c file generated by New Project wizard
 *
 * Created:   Mon Feb 5 2024
 * Processor: PIC24FJ128GA006
 * Compiler:  MPLAB XC16
 */

#include <p24FJ128GA006.h>
#include "SEE.h"

_CONFIG2( IESO_OFF          // two speed start up disabled
        & FCKSM_CSDCMD      // disable clock-swithcing/monitor
        & FNOSC_FRCPLL      // Fast oscillator: enable PLL 
        & POSCMOD_NONE)       // primary oscillator: internal mode

#include "LCD.H"

#define F_KEY 		1
#define DOWN_KEY 	2
#define TARE_KEY 	3
#define LH_KEY 		4
#define TEST_KEY 	5
#define DUTY_KEY 	6
#define ALT_KEY 	7
#define UP_KEY 		8
#define DED_KEY 	9

#define	NORMAL	0
#define	FAULT	1
#define	ALT		2
#define	CAL		3

#define AINPUTS 0xfffe  	// Analog inputs for ADC ch 0

struct INOUT
	{
	unsigned EXEC:1;
	unsigned SENSE:1;
	unsigned ALM :1;
	}digital[8];

unsigned int analogue[8];
char firstkey = 0, secondkey = 0, lastkey=0;
char text[] = {"Hello wolrd!"};
unsigned char MODE=0;


void delayUs(int x)
	{
	while(x--);		
	}

void delayMs(int x)
	{
	while(x--)
		delayUs(1000);
	}

// initialize the ADC for single conversion, select Analog input pins
void initADC( int amask) 
{
    AD1PCFG = amask;    // select analog input pins
    AD1CON1 = 0x00E0;   // auto convert after end of sampling 
    AD1CSSL = 0;        // no scanning required 
    AD1CON3 = 0x1F02;   // max sample time = 31Tad, Tad = 2 x Tcy = 125ns >75ns
    AD1CON2 = 0;        // use MUXA, AVss and AVdd are used as Vref+/-
    AD1CON1bits.ADON = 1; // turn on the ADC
} //initADC

void beep(void)
	{
	_RD6 = 1;
	delayMs(250);
	_RD6 = 0;
	delayMs(250);
	}

int readADC( int ch)
{
    AD1CHS  = ch;               // select analog input channel
    AD1CON1bits.SAMP = 1;       // start sampling, automatic conversion will follow
    while (!AD1CON1bits.DONE);   // wait to complete the conversion
    return ADC1BUF0;            // read the conversion result
} // readADC

int readKeypad(void)
	{
	int ch , tally=0;
	int key = 0;

	for(ch = 1;ch < 8; ch<<=1)
		{
		LATD &= 0xFFF8;
		LATD |= ch;
		if(_RB4) key+=1;
		if(_RB5) key+=2;
		if(_RB6) key+=3;
		if(key) 
			{
			lastkey = secondkey;
			secondkey = firstkey;
			firstkey = key + tally;
			beep();
			return 1;
			}
		tally+=3;
		}
	return 0;
	}

void chipInit(void)
	{
	CLKDIV = 0x8100;
	initADC( AINPUTS);  // initialize the ADC for the analog inputs
	TRISB = 0xFD;     	// select the PORTB pins as inputs to read the sensor inputs
	_RD6 = 0;			// buzzer off
	}

int readSwitches(int ch)
	{
	LATD &= 0xFFF8;
	LATD |= ch;
	analogue[ch] = readADC(0);
	digital[ch].EXEC = _RB3;
	digital[ch].SENSE = _RB2;
	_RB1 = digital[ch].ALM;	
	}

int main (void)
 { 
	int x, key;
	chipInit();
	initLCD();
	InitSEE();
	
	iWriteSEE(0x4, 0x48);
	while (1)
		{
		homeLCD();
		putsLCD(text);
		key = readKeypad();
		if(key) 
			{
			if(firstkey == ALT_KEY)
				{
				clrLCD();
				if(secondkey == TEST_KEY)
					{
					if(lastkey == TEST_KEY)
						MODE = CAL;
					else
						MODE = FAULT;
					}
				else
					MODE = ALT;
				delayMs(2000);
				}
			}
		for(x=0;x<8;x++)
			readSwitches(x);
		}
   return 0;
 }   
